



gst-inspect-1.0 --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
gst-launch-1.0  --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
gst-stats-1.0 --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
gst-typefind-1.0 --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %LIBRARY_LIB%\\girepository-1.0\\Gst-1.0.typelib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
